/**
 * Mixed-case SVG and MathML element names recognized in foreign content.
 * @see https://html.spec.whatwg.org/multipage/parsing.html#parsing-main-inforeign
 */
export declare const elementNames: Map<string, string>;
/**
 * Mixed-case SVG and MathML attribute names recognized in foreign content.
 * @see https://html.spec.whatwg.org/multipage/parsing.html#parsing-main-inforeign
 */
export declare const attributeNames: Map<string, string>;
//# sourceMappingURL=foreign-names.d.ts.map