import { type EncodeTrieNode } from "../internal/encode-shared.js";
/** Compact serialized HTML encode trie (intended to stay small & JS engine friendly) */
/** HTML entity encode trie. */
export declare const htmlTrie: Map<number, EncodeTrieNode>;
//# sourceMappingURL=encode-html.d.ts.map