'use strict';
require('../../modules/es.array-buffer.constructor');
require('../../modules/es.array-buffer.slice');
require('../../modules/es.array-buffer.detached');
require('../../modules/es.array-buffer.transfer');
require('../../modules/es.array-buffer.transfer-to-fixed-length');
require('../../modules/es.object.to-string');
var path = require('../../internals/path');

module.exports = path.ArrayBuffer;
